Package: ecoregime
Title: Analysis of Ecological Dynamic Regimes
Version: 0.3.1
Authors@R: 
    person(given = "Martina", family = "Sánchez-Pinillos", 
           email = "martina.sanchez.pinillos@gmail.com", 
           role = c("aut", "cre", "cph"), 
           comment = c(ORCID = "0000-0002-1499-4507"))
Description: A toolbox for implementing the Ecological Dynamic Regime framework 
    (Sánchez-Pinillos et al., 2023 <doi:10.1002/ecm.1589>) to characterize and 
    compare groups of ecological trajectories in multidimensional spaces defined 
    by state variables. The package includes the RETRA-EDR algorithm to identify 
    representative trajectories, functions to generate, summarize, and visualize 
    representative trajectories, and several metrics to quantify the distribution 
    and heterogeneity of trajectories in an ecological dynamic regime and quantify 
    the dissimilarity between two or more ecological dynamic regimes. The package 
    also includes a set of functions to assess ecological resilience based on 
    ecological dynamic regimes (Sánchez-Pinillos et al., 2024 <doi:10.1016/j.biocon.2023.110409>).
License: GPL (>= 3)
Encoding: UTF-8
URL: https://mspinillos.github.io/ecoregime/,
        https://github.com/MSPinillos/ecoregime
BugReports: https://github.com/MSPinillos/ecoregime/issues
Depends: R (>= 4.4.0)
LazyData: true
Imports: ape, data.table, ecotraj (>= 1.1.1), graphics, methods, shape,
        smacof, stats, stringr
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), vegan
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-07 18:54:54 UTC; martina.sanchez
Author: Martina Sánchez-Pinillos [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-1499-4507>)
Maintainer: Martina Sánchez-Pinillos <martina.sanchez.pinillos@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-07 19:30:02 UTC
Built: R 4.6.0; ; 2026-06-07 22:39:07 UTC; unix
