Package: ecoregime
Title: Analysis of Ecological Dynamic Regimes
Version: 0.4.1
Authors@R: 
    person(given = "Martina", family = "Sánchez-Pinillos", 
           email = "martina.sanchez.pinillos@gmail.com", 
           role = c("aut", "cre", "cph"), 
           comment = c(ORCID = "0000-0002-1499-4507"))
Description: A toolbox for implementing the Ecological Dynamic Regime framework, 
    including functions to characterize and compare groups of ecological trajectories 
    (Sánchez-Pinillos et al., 2023 <doi:10.1002/ecm.1589>); assess the ecological 
    resilience of a disturbed system using a reference dynamic regime (Sánchez-Pinillos 
    et al., 2024 <doi:10.1016/j.biocon.2023.110409>); and forecast ecological 
    trajectories from a dynamic regime (Sánchez-Pinillos et al. 2026, 
    <doi:10.1111/2041-210x.70372>). Additional functions are also available for 
    visualizing ecological dynamic regimes, their representative trajectories, 
    as well as predicted trajectories in a multidimensional state space.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://mspinillos.github.io/ecoregime/,
        https://github.com/MSPinillos/ecoregime
BugReports: https://github.com/MSPinillos/ecoregime/issues
Depends: R (>= 4.4.0)
LazyData: true
Imports: ape, data.table, ecotraj (>= 1.1.1), graphics, grDevices,
        methods, shape, smacof, stats, stringr
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), vegan
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
RoxygenNote: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-04 18:00:52 UTC; martina.sanchez
Author: Martina Sánchez-Pinillos [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-1499-4507>)
Maintainer: Martina Sánchez-Pinillos <martina.sanchez.pinillos@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-04 20:10:02 UTC
Built: R 4.6.0; ; 2026-08-04 23:36:27 UTC; unix
