# Generated by roxygen2: do not edit by hand

S3method(plot,RETRA)
S3method(summary,RETRA)
export(amplitude)
export(dBD)
export(dDis)
export(dEve)
export(define_retra)
export(dist_edr)
export(net_change)
export(plot_edr)
export(recovery)
export(resistance)
export(retra_edr)
export(state_to_trajectory)
importFrom(data.table,":=")
importFrom(data.table,.BY)
importFrom(data.table,.EACHI)
importFrom(data.table,.GRP)
importFrom(data.table,.I)
importFrom(data.table,.N)
importFrom(data.table,.NGRP)
importFrom(data.table,.SD)
importFrom(data.table,data.table)
importFrom(graphics,lines)
importFrom(methods,is)
importFrom(stats,as.dist)
importFrom(stats,dist)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(stats,weighted.mean)
